@@ -43,23 +43,20 @@ gl_err_to_string(GLenum err)
     }
 }
 
-
-#define GLERRPRINT(msg)                                                     \
-    for (GLenum err = glGetError(); err != GL_NO_ERROR; err = glGetError()) \
-        std::cerr << "GL error " << msg << " " << (int)err << " - "         \
-                  << gl_err_to_string(err) << "\n";
-
-
-
 IvGL::IvGL(QWidget* parent, ImageViewer& viewer)
     : QOpenGLWidget(parent)
     , m_viewer(viewer)
     , m_shaders_created(false)
+    , m_vertex_shader(0)
+    , m_shader_program(0)
     , m_tex_created(false)
     , m_zoom(1.0)
     , m_centerx(0)
     , m_centery(0)
     , m_dragging(false)
+    , m_mousex(0)
+    , m_mousey(0)
+    , m_drag_button(Qt::NoButton)
     , m_use_shaders(false)
     , m_use_halffloat(false)
     , m_use_float(false)
@@ -151,19 +148,19 @@ IvGL::create_textures(void)
     for (unsigned int texture : textures) {
         m_texbufs.emplace_back();
         glBindTexture(GL_TEXTURE_2D, texture);
-        GLERRPRINT("bind tex");
+        print_error("bind tex");
         glTexImage2D(GL_TEXTURE_2D, 0 /*mip level*/,
                      4 /*internal format - color components */, 1 /*width*/,
                      1 /*height*/, 0 /*border width*/,
                      GL_RGBA /*type - GL_RGB, GL_RGBA, GL_LUMINANCE */,
                      GL_FLOAT /*format - GL_FLOAT */, NULL /*data*/);
-        GLERRPRINT("tex image 2d");
+        print_error("tex image 2d");
         // Initialize tex parameters.
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
-        GLERRPRINT("After tex parameters");
+        print_error("After tex parameters");
         m_texbufs.back().tex_object = texture;
         m_texbufs.back().x          = 0;
         m_texbufs.back().y          = 0;
@@ -189,25 +186,80 @@ IvGL::create_textures(void)
     m_tex_created = true;
 }
 
-
+const char*
+IvGL::color_func_shader_text()
+{
+    // clang-format off
+    return
+        "uniform float gain;\n"
+        "uniform float gamma;\n"
+        "\n"
+        "vec4 ColorFunc(vec4 C)\n"
+        "{\n"
+        "    C.xyz *= gain;\n"
+        "    float invgamma = 1.0/gamma;\n"
+        "    C.xyz = pow (C.xyz, vec3 (invgamma, invgamma, invgamma));\n"
+        "    return C;\n"
+        "}\n";
+    // clang-format on
+}
 
 void
 IvGL::create_shaders(void)
 {
-    // clang-format off
-    static const GLchar *vertex_source =
-        "varying vec2 vTexCoord;\n"
-        "void main ()\n"
-        "{\n"
-        "    vTexCoord = gl_MultiTexCoord0.xy;\n"
-        "    gl_Position = ftransform();\n"
-        "}\n";
+    if (!m_use_shaders) {
+        std::cerr << "Not using shaders!\n";
+        return;
+    }
 
-    static const GLchar *fragment_source =
+    const char* color_shader = color_func_shader_text();
+    if (m_color_shader_text != color_shader) {
+        if (m_shader_program) {
+            if (m_vertex_shader) {
+                glDetachShader(m_shader_program, m_vertex_shader);
+            }
+            glUseProgram(0);
+            glDeleteProgram(m_shader_program);
+            m_shader_program  = 0;
+            m_shaders_created = false;
+        }
+    }
+
+    if (m_shaders_created) {
+        return;
+    }
+
+    // This holds the compilation status
+    GLint status;
+
+    if (!m_vertex_shader) {
+        // clang-format off
+        static const GLchar* vertex_source =
+            "varying vec2 vTexCoord;\n"
+            "void main ()\n"
+            "{\n"
+            "    vTexCoord = gl_MultiTexCoord0.xy;\n"
+            "    gl_Position = ftransform();\n"
+            "}\n";
+        // clang-format on
+
+        m_vertex_shader = glCreateShader(GL_VERTEX_SHADER);
+        glShaderSource(m_vertex_shader, 1, &vertex_source, NULL);
+        glCompileShader(m_vertex_shader);
+        glGetShaderiv(m_vertex_shader, GL_COMPILE_STATUS, &status);
+
+        if (!status) {
+            std::cerr << "vertex shader compile status: " << status << "\n";
+            print_shader_log(std::cerr, m_vertex_shader);
+            create_shaders_abort();
+            return;
+        }
+    }
+
+    // clang-format off
+    static const GLchar* fragment_source =
         "uniform sampler2D imgtex;\n"
         "varying vec2 vTexCoord;\n"
-        "uniform float gain;\n"
-        "uniform float gamma;\n"
         "uniform int startchannel;\n"
         "uniform int colormode;\n"
         // Remember, if imgchannels == 2, second channel would be channel 4 (a).
@@ -305,76 +357,60 @@ IvGL::create_shaders(void)
         "        C = heatmap_mode (C);\n"
         "    if (pixelview != 0)\n"
         "        C.a = 1.0;\n"
-        "    C.xyz *= gain;\n"
-        "    float invgamma = 1.0/gamma;\n"
-        "    C.xyz = pow (C.xyz, vec3 (invgamma, invgamma, invgamma));\n"
+        "    C = ColorFunc(C);\n"
         "    gl_FragColor = C;\n"
         "}\n";
     // clang-format on
 
-    if (!m_use_shaders) {
-        std::cerr << "Not using shaders!\n";
-        return;
-    }
-    if (m_shaders_created)
-        return;
+    const char* fragment_sources[] = { "#version 120\n", color_shader,
+                                       fragment_source };
+    m_color_shader_text            = color_shader;
 
-    //initialize shader object handles for abort function
-    m_shader_program  = 0;
-    m_vertex_shader   = 0;
-    m_fragment_shader = 0;
-
-    // When using extensions to support shaders, we need to load the function
-    // entry points (which is actually done by GLEW) and then call them. So
-    // we have to get the functions through the right symbols otherwise
-    // extension-based shaders won't work.
-    m_shader_program = glCreateProgram();
-
-    GLERRPRINT("create program");
-
-    // This holds the compilation status
-    GLint status;
-
-    m_vertex_shader = glCreateShader(GL_VERTEX_SHADER);
-    glShaderSource(m_vertex_shader, 1, &vertex_source, NULL);
-    glCompileShader(m_vertex_shader);
-    glGetShaderiv(m_vertex_shader, GL_COMPILE_STATUS, &status);
-
-    if (!status) {
-        std::cerr << "vertex shader compile status: " << status << "\n";
-        print_shader_log(std::cerr, m_vertex_shader);
-        create_shaders_abort();
-        return;
-    }
-    glAttachShader(m_shader_program, m_vertex_shader);
-    GLERRPRINT("After attach vertex shader.");
-
-    m_fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
-    glShaderSource(m_fragment_shader, 1, &fragment_source, NULL);
-    glCompileShader(m_fragment_shader);
-    glGetShaderiv(m_fragment_shader, GL_COMPILE_STATUS, &status);
+    GLuint fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
+    glShaderSource(fragment_shader, 3, fragment_sources, NULL);
+    glCompileShader(fragment_shader);
+    glGetShaderiv(fragment_shader, GL_COMPILE_STATUS, &status);
     if (!status) {
         std::cerr << "fragment shader compile status: " << status << "\n";
-        print_shader_log(std::cerr, m_fragment_shader);
+        print_shader_log(std::cerr, fragment_shader);
         create_shaders_abort();
         return;
     }
-    glAttachShader(m_shader_program, m_fragment_shader);
-    GLERRPRINT("After attach fragment shader");
 
-    glLinkProgram(m_shader_program);
-    GLERRPRINT("link");
-    GLint linked;
-    glGetProgramiv(m_shader_program, GL_LINK_STATUS, &linked);
-    if (!linked) {
-        std::cerr << "NOT LINKED\n";
-        char buf[10000];
-        buf[0] = 0;
-        GLsizei len;
-        glGetProgramInfoLog(m_shader_program, sizeof(buf), &len, buf);
-        std::cerr << "link log:\n" << buf << "---\n";
-        create_shaders_abort();
-        return;
+    if (!m_shader_program) {
+        // When using extensions to support shaders, we need to load the
+        // function entry points (which is actually done by GLEW) and then call
+        // them. So we have to get the functions through the right symbols
+        // otherwise extension-based shaders won't work.
+        m_shader_program = glCreateProgram();
+        print_error("create program");
+
+        glAttachShader(m_shader_program, m_vertex_shader);
+        print_error("After attach vertex shader.");
+
+        glAttachShader(m_shader_program, fragment_shader);
+        print_error("After attach fragment shader");
+
+        glLinkProgram(m_shader_program);
+        print_error("link");
+        GLint linked;
+        glGetProgramiv(m_shader_program, GL_LINK_STATUS, &linked);
+        if (!linked) {
+            std::cerr << "NOT LINKED\n";
+            char buf[10000];
+            buf[0] = 0;
+            GLsizei len;
+            glGetProgramInfoLog(m_shader_program, sizeof(buf), &len, buf);
+            std::cerr << "link log:\n" << buf << "---\n";
+            create_shaders_abort();
+            return;
+        }
+
+        glDetachShader(m_shader_program, fragment_shader);
+        print_error("After detach fragment shader");
+
+        glDeleteShader(fragment_shader);
+        print_error("After delete fragment shader");
     }
 
     m_shaders_created = true;
@@ -390,10 +426,8 @@ IvGL::create_shaders_abort(void)
         glDeleteProgram(m_shader_program);
     if (m_vertex_shader)
         glDeleteShader(m_vertex_shader);
-    if (m_fragment_shader)
-        glDeleteShader(m_fragment_shader);
 
-    GLERRPRINT("After delete shaders");
+    print_error("After delete shaders");
     m_use_shaders = false;
 }
 
@@ -402,7 +436,7 @@ IvGL::create_shaders_abort(void)
 void
 IvGL::resizeGL(int w, int h)
 {
-    GLERRPRINT("resizeGL entry");
+    print_error("resizeGL entry");
     glViewport(0, 0, w, h);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
@@ -413,7 +447,7 @@ IvGL::resizeGL(int w, int h)
     glMatrixMode(GL_MODELVIEW);
 
     clamp_view_to_window();
-    GLERRPRINT("resizeGL exit");
+    print_error("resizeGL exit");
 }
 
 
@@ -438,6 +472,32 @@ gl_rect(float xmin, float ymin, float xmax, float ymax, float z = 0,
 
 
 
+static void
+gl_rect_border(float xmin, float ymin, float xmax, float ymax, float z = 0)
+{
+    glBegin(GL_LINE_LOOP);
+    glVertex3f(xmin, ymin, z);
+    glVertex3f(xmax, ymin, z);
+    glVertex3f(xmax, ymax, z);
+    glVertex3f(xmin, ymax, z);
+    glEnd();
+}
+
+
+
+static void
+gl_rect_dotted_border(float xmin, float ymin, float xmax, float ymax,
+                      float z = 0)
+{
+    glPushAttrib(GL_ENABLE_BIT);
+    glLineStipple(1, 0xF0F0);
+    glEnable(GL_LINE_STIPPLE);
+    gl_rect_border(xmin, ymin, xmax, ymax, z);
+    glPopAttrib();
+}
+
+
+
 static void
 handle_orientation(int orientation, int width, int height, float& scale_x,
                    float& scale_y, float& rotate_z, float& point_x,
@@ -550,6 +610,8 @@ IvGL::paintGL()
     // Recentered so that the pixel space (m_centerx,m_centery) position is
     // at the center of the visible window.
 
+    update_state();
+
     useshader(m_texture_width, m_texture_height);
 
     float smin = 0, smax = 1.0;
@@ -598,6 +660,10 @@ IvGL::paintGL()
         }
     }
 
+    if (m_viewer.windowguidesOn()) {
+        paint_windowguides();
+    }
+
     glPopMatrix();
 
     if (m_viewer.pixelviewOn()) {
@@ -624,7 +690,9 @@ IvGL::shadowed_text(float x, float y, float /*z*/, const std::string& s,
      * Paint on intermediate QImage, AA text on QOpenGLWidget based
      * QPaintDevice requires MSAA
      */
-    QImage t(size(), QImage::Format_ARGB32_Premultiplied);
+    qreal dpr = devicePixelRatio();
+    QImage t(size() * dpr, QImage::Format_ARGB32_Premultiplied);
+    t.setDevicePixelRatio(dpr);
     t.fill(qRgba(0, 0, 0, 0));
     {
         QPainter painter(&t);
@@ -754,9 +822,10 @@ IvGL::paint_pixelview()
                                      m_viewer.current_color_mode());
         }
 
-        void* zoombuffer = OIIO_ALLOCA(char, (xend - xbegin) * (yend - ybegin)
-                                                 * nchannels
-                                                 * spec.channel_bytes());
+        auto zoombuffer = OIIO_ALLOCA_SPAN(std::byte,
+                                           (xend - xbegin) * (yend - ybegin)
+                                               * nchannels
+                                               * spec.channel_bytes());
         if (!m_use_shaders) {
             img->get_pixels(ROI(spec.x + xbegin, spec.x + xend, spec.y + ybegin,
                                 spec.y + yend),
@@ -774,8 +843,8 @@ IvGL::paint_pixelview()
         glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
         glBindTexture(GL_TEXTURE_2D, m_pixelview_tex);
         glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, xend - xbegin, yend - ybegin,
-                        glformat, gltype, zoombuffer);
-        GLERRPRINT("After tsi2d");
+                        glformat, gltype, zoombuffer.data());
+        print_error("After tsi2d");
     } else {
         smin = -1;
         smax = -1;
@@ -835,8 +904,8 @@ IvGL::paint_pixelview()
                 texty = closeupsize + yspacing;
             }
         }
-        std::string s = Strutil::sprintf("(%d, %d)", (int)real_xp + spec.x,
-                                         (int)real_yp + spec.y);
+        std::string s = Strutil::fmt::format("({}, {})", (int)real_xp + spec.x,
+                                             (int)real_yp + spec.y);
         shadowed_text(textx, texty, 0.0f, s, font);
         texty += yspacing;
         img->getpixel((int)real_xp + spec.x, (int)real_yp + spec.y, fpixel);
@@ -845,20 +914,20 @@ IvGL::paint_pixelview()
             case TypeDesc::UINT8: {
                 ImageBuf::ConstIterator<unsigned char, unsigned char> p(
                     *img, (int)real_xp + spec.x, (int)real_yp + spec.y);
-                s = Strutil::sprintf("%s: %3d  (%5.3f)",
-                                     spec.channelnames[i].c_str(), (int)(p[i]),
-                                     fpixel[i]);
+                s = Strutil::fmt::format("{}: {:3}  ({:5.3f})",
+                                         spec.channelnames[i], int(p[i]),
+                                         fpixel[i]);
             } break;
             case TypeDesc::UINT16: {
                 ImageBuf::ConstIterator<unsigned short, unsigned short> p(
                     *img, (int)real_xp + spec.x, (int)real_yp + spec.y);
-                s = Strutil::sprintf("%s: %3d  (%5.3f)",
-                                     spec.channelnames[i].c_str(), (int)(p[i]),
-                                     fpixel[i]);
+                s = Strutil::fmt::format("{}: {:3}  ({:5.3f})",
+                                         spec.channelnames[i], int(p[i]),
+                                         fpixel[i]);
             } break;
             default:  // everything else, treat as float
-                s = Strutil::sprintf("%s: %5.3f", spec.channelnames[i].c_str(),
-                                     fpixel[i]);
+                s = Strutil::fmt::format("{}: {:5.3f}", spec.channelnames[i],
+                                         fpixel[i]);
             }
             shadowed_text(textx, texty, 0.0f, s, font);
             texty += yspacing;
@@ -871,13 +940,44 @@ IvGL::paint_pixelview()
 
 
 
+void
+IvGL::paint_windowguides()
+{
+    IvImage* img = m_current_image;
+    const ImageSpec& spec(img->spec());
+
+    glDisable(GL_TEXTURE_2D);
+    glUseProgram(0);
+    glPushAttrib(GL_ENABLE_BIT);
+    glEnable(GL_COLOR_LOGIC_OP);
+    glLogicOp(GL_XOR);
+
+    // Data window
+    {
+        const float xmin = spec.x;
+        const float xmax = spec.x + spec.width;
+        const float ymin = spec.y;
+        const float ymax = spec.y + spec.height;
+        gl_rect_border(xmin, ymin, xmax, ymax);
+    }
+
+    // Display window
+    {
+        const float xmin = spec.full_x;
+        const float xmax = spec.full_x + spec.full_width;
+        const float ymin = spec.full_y;
+        const float ymax = spec.full_y + spec.full_height;
+        gl_rect_dotted_border(xmin, ymin, xmax, ymax);
+    }
+
+    glPopAttrib();
+}
+
+
+
 void
 IvGL::useshader(int tex_width, int tex_height, bool pixelview)
 {
-    IvImage* img = m_viewer.cur();
-    if (!img)
-        return;
-
     if (!m_use_shaders) {
         glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
         for (auto&& tb : m_texbufs) {
@@ -897,10 +997,25 @@ IvGL::useshader(int tex_width, int tex_height, bool pixelview)
         return;
     }
 
-    const ImageSpec& spec(img->spec());
+    use_program();
+    update_uniforms(tex_width, tex_height, pixelview);
+}
 
+void
+IvGL::use_program(void)
+{
     glUseProgram(m_shader_program);
-    GLERRPRINT("After use program");
+    print_error("After use program");
+}
+
+void
+IvGL::update_uniforms(int tex_width, int tex_height, bool pixelview)
+{
+    IvImage* img = m_viewer.cur();
+    if (!img)
+        return;
+
+    const ImageSpec& spec(img->spec());
 
     GLint loc;
 
@@ -915,8 +1030,7 @@ IvGL::useshader(int tex_width, int tex_height, bool pixelview)
     // This is the texture unit, not the texture object
     glUniform1i(loc, 0);
 
-    loc = glGetUniformLocation(m_shader_program, "gain");
-
+    loc        = glGetUniformLocation(m_shader_program, "gain");
     float gain = powf(2.0, img->exposure());
     glUniform1f(loc, gain);
 
@@ -940,7 +1054,8 @@ IvGL::useshader(int tex_width, int tex_height, bool pixelview)
 
     loc = glGetUniformLocation(m_shader_program, "height");
     glUniform1i(loc, tex_height);
-    GLERRPRINT("After setting uniforms");
+
+    print_error("After setting uniforms");
 }
 
 
@@ -986,14 +1101,14 @@ IvGL::update()
         glTexImage2D(GL_TEXTURE_2D, 0 /*mip level*/, glinternalformat,
                      m_texture_width, m_texture_height, 0 /*border width*/,
                      glformat, gltype, NULL /*data*/);
-        GLERRPRINT("Setting up texture");
+        print_error("Setting up texture");
     }
 
     // Set the right type for the texture used for pixelview.
     glBindTexture(GL_TEXTURE_2D, m_pixelview_tex);
     glTexImage2D(GL_TEXTURE_2D, 0, glinternalformat, closeuptexsize,
                  closeuptexsize, 0, glformat, gltype, NULL);
-    GLERRPRINT("Setting up pixelview texture");
+    print_error("Setting up pixelview texture");
 
     // Resize the buffer at once, rather than create one each drawing.
     m_tex_buffer.resize(m_texture_width * m_texture_height * nchannels
@@ -1105,13 +1220,13 @@ IvGL::mousePressEvent(QMouseEvent* event)
         switch (event->button()) {
         case Qt::LeftButton:
             if (mousemode == ImageViewer::MouseModeZoom && !Alt)
-                m_viewer.zoomIn();
+                m_viewer.zoomIn(true);  // Animated zoom for mouse clicks
             else
                 m_dragging = true;
             return;
         case Qt::RightButton:
             if (mousemode == ImageViewer::MouseModeZoom && !Alt)
-                m_viewer.zoomOut();
+                m_viewer.zoomOut(true);  // Animated zoom for mouse clicks
             else
                 m_dragging = true;
             return;
@@ -1215,9 +1330,11 @@ IvGL::wheelEvent(QWheelEvent* event)
     QPoint angdelta    = event->angleDelta() / 8;  // div by 8 to get degrees
     if (abs(angdelta.y()) > abs(angdelta.x())      // predominantly vertical
         && abs(angdelta.y()) > 2) {                // suppress tiny motions
-        float oldzoom = m_viewer.zoom();
-        float newzoom = (angdelta.y() > 0) ? ceil2f(oldzoom) : floor2f(oldzoom);
-        m_viewer.zoom(newzoom);
+        if (angdelta.y() > 0) {
+            m_viewer.zoomIn(false);
+        } else {
+            m_viewer.zoomOut(false);
+        }
         event->accept();
     }
     // TODO: Update this to keep the zoom centered on the event .x, .y
@@ -1485,12 +1602,14 @@ IvGL::load_texture(int x, int y, int width, int height)
     // may not be resident at once.
     if (!m_use_shaders) {
         m_current_image->get_pixels(ROI(x, x + width, y, y + height),
-                                    spec.format, &m_tex_buffer[0]);
+                                    spec.format,
+                                    as_writable_bytes(make_span(m_tex_buffer)));
     } else {
         m_current_image->get_pixels(ROI(x, x + width, y, y + height, 0, 1,
                                         m_viewer.current_channel(),
                                         m_viewer.current_channel() + nchannels),
-                                    spec.format, &m_tex_buffer[0]);
+                                    spec.format,
+                                    as_writable_bytes(make_span(m_tex_buffer)));
     }
 
     glBindBuffer(GL_PIXEL_UNPACK_BUFFER, m_pbo_objects[m_last_pbo_used]);
@@ -1499,17 +1618,17 @@ IvGL::load_texture(int x, int y, int width, int height)
                             * uint64_t(nchannels)
                             * uint64_t(spec.format.size())),
                  &m_tex_buffer[0], GL_STREAM_DRAW);
-    GLERRPRINT("After buffer data");
+    print_error("After buffer data");
     m_last_pbo_used = (m_last_pbo_used + 1) & 1;
 
     // When using PBO this is the offset within the buffer.
     void* data = 0;
 
     glBindTexture(GL_TEXTURE_2D, tb.tex_object);
-    GLERRPRINT("After bind texture");
+    print_error("After bind texture");
     glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, width, height, glformat, gltype,
                     data);
-    GLERRPRINT("After loading sub image");
+    print_error("After loading sub image");
     m_last_texbuf_used = (m_last_texbuf_used + 1) % m_texbufs.size();
     glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
 }
@@ -1524,5 +1643,18 @@ IvGL::is_too_big(float width, float height)
     return tiles > m_texbufs.size();
 }
 
+void
+IvGL::update_state(void)
+{
+    create_shaders();
+}
+
+void
+IvGL::print_error(const char* msg)
+{
+    for (GLenum err = glGetError(); err != GL_NO_ERROR; err = glGetError())
+        std::cerr << "GL error " << msg << " " << (int)err << " - "
+                  << gl_err_to_string(err) << "\n";
+}
 
 OIIO_PRAGMA_WARNING_POP
