@@ -379,14 +379,16 @@ private:
                                   int alpha_channel, TypeDesc format) const;
 
     template<typename T>
-    static void cmyk_to_rgb(int n, const T* cmyk, size_t cmyk_stride, T* rgb,
-                            size_t rgb_stride)
+    static void cmyk_to_rgb(int n, cspan<T> cmyk, size_t cmyk_stride,
+                            span<T> rgb, size_t rgb_stride)
     {
-        for (; n; --n, cmyk += cmyk_stride, rgb += rgb_stride) {
-            float C = convert_type<T, float>(cmyk[0]);
-            float M = convert_type<T, float>(cmyk[1]);
-            float Y = convert_type<T, float>(cmyk[2]);
-            float K = convert_type<T, float>(cmyk[3]);
+        OIIO_DASSERT(size_t(n) * cmyk_stride <= std::size(cmyk));
+        OIIO_DASSERT(size_t(n) * rgb_stride <= std::size(rgb));
+        for (int i = 0; i < n; ++i) {
+            float C = convert_type<T, float>(cmyk[i * cmyk_stride + 0]);
+            float M = convert_type<T, float>(cmyk[i * cmyk_stride + 1]);
+            float Y = convert_type<T, float>(cmyk[i * cmyk_stride + 2]);
+            float K = convert_type<T, float>(cmyk[i * cmyk_stride + 3]);
 #if 0
             // WHY doesn't this work if it's cmyk?
             float R = (1.0f - C) * (1.0f - K);
@@ -400,12 +402,13 @@ private:
             float G = M * (K);
             float B = Y * (K);
 #endif
-            rgb[0] = convert_type<float, T>(R);
-            rgb[1] = convert_type<float, T>(G);
-            rgb[2] = convert_type<float, T>(B);
+            rgb[i * rgb_stride + 0] = convert_type<float, T>(R);
+            rgb[i * rgb_stride + 1] = convert_type<float, T>(G);
+            rgb[i * rgb_stride + 2] = convert_type<float, T>(B);
 
             if (cmyk_stride == 5 && rgb_stride == 4) {
-                rgb[3] = convert_type<float, T>(cmyk[4]);
+                rgb[i * rgb_stride + 3] = convert_type<float, T>(
+                    cmyk[i * cmyk_stride + 4]);
             }
         }
     }
@@ -505,8 +508,8 @@ const PSDInput::ResourceLoader PSDInput::resource_loaders[]
 
 
 const char* PSDInput::additional_info_psb[]
-    = { "LMsk", "Lr16", "Lr32", "Layr", "Mt16", "Mt32", "Mtrn",
-        "Alph", "FMsk", "Ink2", "FEid", "FXid", "PxSD", "cinf" };
+    = { "LMsk", "Lr16", "Lr32", "Layr", "Mt16", "Mt32", "Mtrn", "Alph",
+        "FMsk", "lnk2", "FEid", "FXid", "PxSD", "cinf", "lnkE", "pths" };
 
 const unsigned int PSDInput::additional_info_psb_count
     = sizeof(additional_info_psb) / sizeof(additional_info_psb[0]);
@@ -545,17 +548,6 @@ OIIO_PLUGIN_EXPORTS_END
 
 
 
-/// Byteswap a span of values.  This should work for any of short, unsigned
-/// short, int, unsigned int, float, long long, pointers. It's just calling
-/// byteswap on each element.
-template<typename T>
-inline OIIO_HOSTDEVICE void
-byteswap_span(span<T> vals)
-{
-    swap_endian(vals.data(), int(vals.size()));
-}
-
-
 PSDInput::PSDInput() { init(); }
 
 
@@ -578,6 +570,7 @@ PSDInput::open(const std::string& name, ImageSpec& newspec)
     m_filename = name;
     if (!ioproxy_use_or_open(name))
         return false;
+    ioseek(0);
 
     // File Header
     if (!load_header()) {
@@ -814,41 +807,47 @@ PSDInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
             break;
         }
     } else if (m_header.color_mode == ColorMode_CMYK) {
+        span_size_t cmyklen = channel_count * spec.width;
         switch (bps) {
         case 4: {
-            std::unique_ptr<float[]> cmyk(
-                new float[channel_count * spec.width]);
+            std::unique_ptr<float[]> cmyk(new float[cmyklen]);
             interleave_row(cmyk.get(), channel_buffers, spec.width,
                            channel_count);
-            cmyk_to_rgb(spec.width, cmyk.get(), channel_count, (float*)dst,
+            cmyk_to_rgb(spec.width, make_cspan(cmyk.get(), cmyklen),
+                        channel_count,
+                        make_span((float*)dst, spec.width * spec.nchannels),
                         spec.nchannels);
             break;
         }
         case 2: {
-            std::unique_ptr<unsigned short[]> cmyk(
-                new unsigned short[channel_count * spec.width]);
+            std::unique_ptr<unsigned short[]> cmyk(new unsigned short[cmyklen]);
             interleave_row(cmyk.get(), channel_buffers, spec.width,
                            channel_count);
-            cmyk_to_rgb(spec.width, cmyk.get(), channel_count,
-                        (unsigned short*)dst, spec.nchannels);
+            cmyk_to_rgb(spec.width, make_cspan(cmyk.get(), cmyklen),
+                        channel_count,
+                        make_span((uint16_t*)dst, spec.width * spec.nchannels),
+                        spec.nchannels);
             break;
         }
         default: {
-            std::unique_ptr<unsigned char[]> cmyk(
-                new unsigned char[channel_count * spec.width]);
+            std::unique_ptr<unsigned char[]> cmyk(new unsigned char[cmyklen]);
             interleave_row(cmyk.get(), channel_buffers, spec.width,
                            channel_count);
-            cmyk_to_rgb(spec.width, cmyk.get(), channel_count,
-                        (unsigned char*)dst, spec.nchannels);
+            cmyk_to_rgb(spec.width, make_cspan(cmyk.get(), cmyklen),
+                        channel_count,
+                        make_span((uint8_t*)dst, spec.width * spec.nchannels),
+                        spec.nchannels);
             break;
         }
         }
     } else if (m_header.color_mode == ColorMode_Indexed) {
-        if (!indexed_to_rgb({ (unsigned char*)dst, spec.width * spec.nchannels },
+        if (!indexed_to_rgb({ (unsigned char*)dst,
+                              span_size_t(spec.width * spec.nchannels) },
                             channel_buffers[0], spec.width))
             return false;
     } else if (m_header.color_mode == ColorMode_Bitmap) {
-        if (!bitmap_to_rgb({ (unsigned char*)dst, spec.width * spec.nchannels },
+        if (!bitmap_to_rgb({ (unsigned char*)dst,
+                             span_size_t(spec.width * spec.nchannels) },
                            channel_buffers[0], spec.width))
             return false;
     } else {
@@ -1246,8 +1245,15 @@ PSDInput::load_resource_1039(uint32_t length)
     TypeDesc type(TypeDesc::UINT8, length);
     common_attribute("ICCProfile", type, icc_buf.get());
     std::string errormsg;
-    decode_icc_profile(cspan<uint8_t>(icc_buf.get(), length), m_common_attribs,
-                       errormsg);
+    bool ok = decode_icc_profile(cspan<uint8_t>(icc_buf.get(), length),
+                                 m_common_attribs, errormsg)
+              && decode_icc_profile(cspan<uint8_t>(icc_buf.get(), length),
+                                    m_composite_attribs, errormsg);
+    if (!ok && OIIO::get_int_attribute("imageinput:strict")) {
+        errorfmt("Possible corrupt file, could not decode ICC profile: {}\n",
+                 errormsg);
+        return false;
+    }
     return true;
 }
 
@@ -1892,6 +1898,8 @@ PSDInput::load_image_data()
     // setup some generic properties and read any RLE lengths
     // Image Data Section has RLE lengths for all channels stored first
     for (ChannelInfo& channel_info : m_image_data.channel_info) {
+        channel_info.width       = m_header.width;
+        channel_info.height      = m_header.height;
         channel_info.compression = compression;
         channel_info.channel_id  = id++;
         channel_info.data_length = row_length * m_header.height;
@@ -2047,39 +2055,39 @@ PSDInput::read_channel_row(ChannelInfo& channel_info, uint32_t row, char* data)
             case 16: swap_endian((uint16_t*)data, channel_info.width); break;
             case 32: swap_endian((uint32_t*)data, channel_info.width); break;
             }
-            break;
-        case Compression_RLE: {
-            if (!ioseek(channel_info.row_pos[row]))
-                return false;
-            uint32_t rle_length = channel_info.rle_lengths[row];
-            char* rle_buffer;
-            OIIO_ALLOCATE_STACK_OR_HEAP(rle_buffer, char, rle_length);
-            if (!ioread(rle_buffer, rle_length)
-                || !decompress_packbits(rle_buffer, data, rle_length,
-                                        channel_info.row_length))
-                return false;
-        } break;
-        case Compression_ZIP: {
-            OIIO_ASSERT(channel_info.decompressed_data.size()
-                        == static_cast<uint64_t>(channel_info.width)
-                               * channel_info.height * (m_header.depth / 8));
-            // We simply copy over the row into destination
-            uint64_t row_index = static_cast<uint64_t>(row) * channel_info.width
-                                 * (m_header.depth / 8);
-            std::memcpy(data, channel_info.decompressed_data.data() + row_index,
-                        channel_info.row_length);
-        } break;
-        case Compression_ZIP_Predict: {
-            OIIO_ASSERT(channel_info.decompressed_data.size()
-                        == static_cast<uint64_t>(channel_info.width)
-                               * channel_info.height * (m_header.depth / 8));
-            // We simply copy over the row into destination
-            uint64_t row_index = static_cast<uint64_t>(row) * channel_info.width
-                                 * (m_header.depth / 8);
-            std::memcpy(data, channel_info.decompressed_data.data() + row_index,
-                        channel_info.row_length);
-        } break;
         }
+        break;
+    case Compression_RLE: {
+        if (!ioseek(channel_info.row_pos[row]))
+            return false;
+        uint32_t rle_length = channel_info.rle_lengths[row];
+        char* rle_buffer;
+        OIIO_ALLOCATE_STACK_OR_HEAP(rle_buffer, char, rle_length);
+        if (!ioread(rle_buffer, rle_length)
+            || !decompress_packbits(rle_buffer, data, rle_length,
+                                    channel_info.row_length))
+            return false;
+    } break;
+    case Compression_ZIP: {
+        OIIO_ASSERT(channel_info.decompressed_data.size()
+                    == static_cast<uint64_t>(channel_info.width)
+                           * channel_info.height * (m_header.depth / 8));
+        // We simply copy over the row into destination
+        uint64_t row_index = static_cast<uint64_t>(row) * channel_info.width
+                             * (m_header.depth / 8);
+        std::memcpy(data, channel_info.decompressed_data.data() + row_index,
+                    channel_info.row_length);
+    } break;
+    case Compression_ZIP_Predict: {
+        OIIO_ASSERT(channel_info.decompressed_data.size()
+                    == static_cast<uint64_t>(channel_info.width)
+                           * channel_info.height * (m_header.depth / 8));
+        // We simply copy over the row into destination
+        uint64_t row_index = static_cast<uint64_t>(row) * channel_info.width
+                             * (m_header.depth / 8);
+        std::memcpy(data, channel_info.decompressed_data.data() + row_index,
+                    channel_info.row_length);
+    } break;
     }
 
     return true;
@@ -2232,6 +2240,7 @@ PSDInput::decompress_packbits(const char* src, char* dst,
     int16_t header;
     int length;
 
+    char* dst_start = dst;
     while (src_remaining > 0 && dst_remaining > 0) {
         header = *reinterpret_cast<const signed char*>(src);
         src++;
@@ -2274,8 +2283,8 @@ PSDInput::decompress_packbits(const char* src, char* dst,
 
     if (!bigendian()) {
         switch (m_header.depth) {
-        case 16: swap_endian((uint16_t*)dst, m_spec.width); break;
-        case 32: swap_endian((uint32_t*)dst, m_spec.width); break;
+        case 16: swap_endian((uint16_t*)dst_start, m_spec.width); break;
+        case 32: swap_endian((uint32_t*)dst_start, m_spec.width); break;
         }
     }
 
